# -*- coding: utf-8 -*-
"""
This module contains the generation of apache configuration
"""

import re
import os
import zc.buildout
from Cheetah.Template import Template

#plone.org:127.0.0.1:3128
BACKEND_PATTERN =  re.compile("([^:]*):([^:]*):([0-9]+)")
VHM_PATTERN = re.compile("([^:]*):/(.*)")
VHM_PATH = re.compile("([^/]*)/(.*)")
BIND_PATTERN = re.compile("([^:]*):([0-9]+)$")
NUMBER_PATTERN = re.compile("^([0-9]+)$")
RECIPE_BUILD_NAME = 'plone.recipe.apache:build'


current_dir = os.path.dirname(__file__)

def _get_location(buildout):
    """ get the name of config file """
    for part in buildout.keys():

        if buildout[part].has_key('recipe') and \
            buildout[part]['recipe'] == RECIPE_BUILD_NAME:

            return os.path.join(buildout[part]['location'],'conf','httpd.conf')
    return None


class ConfigureRecipe(object):
    """This recipe is used by zc.buildout
    for building apache
    """

    def __init__(self, buildout, name, options):
        self.name, self.options = name, options

        location = options.get(
            'location', buildout['buildout']['parts-directory'])
        self.options['location'] = location
        self.options['prefix'] = os.path.join(location, name)
        self.options['binary_location'] =  os.path.join(\
                                        buildout['buildout']['directory'],
                                        'bin')

        self.options['mainconfig'] = self.options.get('mainconfig',
                                                      _get_location(buildout)
                                                      )
        self.options['bind'] = str(self.options.get('bind',
                                                '*:80'))

        m = BIND_PATTERN.match(self.options['bind'])
        if m:
            self.options['port'] = '80'
        elif NUMBER_PATTERN.match(self.options['bind'] ):
            self.options['port'] = self.options['bind']
            self.options['bind'] = "*:%s" % (self.options['port'])


        # var directory for cache storage and pid file

        base_dir = buildout['buildout']['directory']
        var_dir = self.options.get('var', os.path.join(base_dir, 'var'))
        # log directory

        log_dir = os.path.join(var_dir, 'log')
        self.options['var_dir'] =var_dir
        self.options['log_dir'] = log_dir
        self.options['log_format'] = self.options.get('log_format','common')

        ## conpute backend

        backends = self.options.get('backends','').split(os.linesep)
        self.backends = []
        for b in backends:
            m = BACKEND_PATTERN.match(b)
            if m:
                (host,backend_host,backend_port) = m.groups()
                vhm = VHM_PATH.match(host)
                if vhm:

                    (host, _vhm_path) =  vhm.groups()
                else:
                    _vhm_path = None



                self.backends.append({'host': host.strip(),
                                      '_vhm_path': _vhm_path and _vhm_path.strip(),
                                      'backend_host' : backend_host,
                                      'backend_port' : backend_port})



        #compute vhm
        vhm_map = self.options.get('zope2_vhm_map','').split(os.linesep)
        self.zope2_vhm_maps = {}
        for vhm in vhm_map:
            #vhm is like plone2.org/plone
            m = VHM_PATTERN.match(vhm)
            if m:
                (host,vhm_path) = m.groups()
                host = host.strip()
                if not self.zope2_vhm_maps.has_key(host):
                    self.zope2_vhm_maps[host] = []
                self.zope2_vhm_maps[host].append(vhm_path.strip())
        ## put all config in this directory
        self.options['config_dir'] =  os.path.join(self.options['prefix'],'conf.d')





    def install(self):
        """installer"""
        self._create_all_directory()
        self._verify_config(self.options['mainconfig'])
        self._make_apacheconf()
        return self.options['config_dir']


    def _create_all_directory(self):
        for dir in ('config_dir','var_dir', 'log_dir'):
            try:
                os.makedirs(self.options[dir])
            except OSError,e:
                ## file exists
                if e.errno == 17:
                    pass
            except WindowsError,e:
                ## file exists
                pass
            except Exception,e:
                ## other raise
                raise e




    def _verify_config(self, config_file):
        """ test if the config is good or not """
        try:
            open(config_file)
        except IOError:
            raise  IOError,"Can not read mainconfig %s" % (config_file,)

    def _make_apacheconf(self):
        """ make apache conf """
        file_name = os.path.join(current_dir,'templates', 'inner_vhost.conf.tmpl')
        tpl = Template(file=file_name)

        for backend in self.backends:

            tpl.backend = backend
            tpl.host = backend['host']
            tpl.zope2_vhm_maps = self.zope2_vhm_maps.get(backend['host'], None)
            tpl.bind = self.options['bind']
            tpl.log_dir = self.options['log_dir']
            tpl.port = self.options['port']
            tpl.log_format = self.options['log_format']
            fd = open(os.path.join(self.options['config_dir'],
                                   "virtual_%s.conf" % (backend['host'],) ),'w')

            print  >>fd, tpl

        fd = open(self.options['mainconfig'])
        content = fd.read()
        fd.close()
        include = "Include %s/*.conf\n" % self.options['config_dir']
        if include not in content:
            fd = open(self.options['mainconfig'],'a')
            fd.write("Include %s/*.conf\n" % self.options['config_dir'])

            fd.close()










import os

import logging, tempfile, setuptools
import urlparse, shutil, urllib2

from zc.recipe.cmmi import Recipe as CMMIRecipe

class BuildRecipe(CMMIRecipe):

    def __init__(self, buildout, name, options):
        #super(BuildRecipe, self).__init__(buildout, name, options)
        CMMIRecipe.__init__(self, buildout, name, options)
        self.name=name
        self.options=options
        self.buildout=buildout
        self.logger=logging.getLogger(self.name)

        self.svn=options.get("svn", None)
        self.url=options.get("url", None)
        if not (self.svn or self.url):
            self.logger.error(
                    "You need to specify either a URL or subversion repository")
            raise zc.buildout.UserError("No download location given")

        # If we use a download, then look for a shared Apache installation directory
        if self.svn is None and buildout['buildout'].get('apache-directory') is not None:
            print 'download already there'
            _, _, urlpath, _, _, _ = urlparse.urlparse(self.url)
            fname = urlpath.split('/')[-1]
            # cleanup the name a bit
            for s in ('.tar', '.bz2', '.gz', '.tgz'):
                fname = fname.replace(s, '')
            location = options['location'] = os.path.join(
                buildout['buildout']['apache-directory'],fname)
            options['shared-apache'] = 'true'
        else:
            # put it into parts
            location = options['location'] = os.path.join(
                buildout['buildout']['parts-directory'],self.name)

        options["source-location"]=os.path.join(location, "source")
        options["binary-location"]=os.path.join(location, "install")
        #options["daemon"]=os.path.join(location, "apache")

        # Set some default options
        buildout['buildout'].setdefault('download-directory',
                os.path.join(buildout['buildout']['directory'], 'downloads'))

    def install(self):
        installed = self.installApache()
        self.addScriptWrappers()
        return installed
        #if self.url and self.options.get('shared-apache') == 'true':
        #    # If the apache installation is shared, only return non-shared paths
        #    return self.options.created()
        #return self.options.created(self.options["location"])

    def installApache(self):
        location=self.options["location"]
        if os.path.exists(location):
            # If the apache installation exists and is shared, then we are done
            if self.options.get('shared-apache') == 'true':
                return
            else:
                shutil.rmtree(location)
        os.mkdir(location)
        self.downloadApache()
        return self.compileApache()
        
    def addScriptWrappers(self):
        bintarget=self.buildout["buildout"]["bin-directory"]

        for dir in ["sbin", "bin"]:
            dir=os.path.join(self.options["location"], dir)
            if not os.path.isdir(dir):
                continue
            if 'apachectl' in os.listdir(dir):
                file = 'apachectl'
                self.logger.info("Adding script wrapper for %s" % file)
                target=os.path.join(bintarget, file)
                f=open(target, "wt")
                print >>f, "#!/bin/sh"
                print >>f, 'exec %s "$@"' % os.path.join(dir, file)
                f.close()
                os.chmod(target, 0755)
                #self.options.created(target)

    def downloadApache(self):
        download_dir=self.buildout['buildout']['download-directory']

        if self.svn:
            self.logger.info("Checking out apache from subversion.")
            assert os.system("svn co %s %s" % (self.options["svn"], self.options["source-location"]))==0
        else:
            self.logger.info("Downloading apache tarball.")
            if not os.path.isdir(download_dir):
                os.mkdir(download_dir)

            _, _, urlpath, _, _, _ = urlparse.urlparse(self.url)
            tmp=tempfile.mkdtemp("buildout-"+self.name)

            try:
                fname=os.path.join(download_dir, urlpath.split("/")[-1])
                if not os.path.exists(fname):
                    f=open(fname, "wb")
                    try:
                        f.write(urllib2.urlopen(self.url).read())
                    except:
                        os.remove(fname)
                        raise
                    f.close()

                setuptools.archive_util.unpack_archive(fname, tmp)

                files=os.listdir(tmp)
                shutil.move(os.path.join(tmp, files[0]), self.options["source-location"])
            finally:
                shutil.rmtree(tmp)

    def compileApache(self):
        os.chdir(self.options["source-location"])
        self.logger.info("Compiling Apache")
        
        extra = """ --with-prefix=%s \
                    --with-included-apr \
                    --enable-headers \
                    --enable-rewrite \
                    --enable-proxy \
                """ % self.options["binary-location"]
                
        modules = self.options.get('modules', '').split(os.linesep)
        for mod in modules:
            mod = mod.strip()
            if mod:
                extra += '--enable-%s ' % mod
                
        self.options['extra_options'] = extra

        # uses cmmi installer
        installed = CMMIRecipe.install(self)
        #installed = '.../plone.recipe.apache/plone/recipe/apache/tests/apache'
        return installed

    def update(self):
        """updater"""
        pass